import Taro from '@tarojs/taro'
import config from '@/base.config'
import {getStore} from '@tarojs/redux';
import {CHANNEL} from '@/constants/base';
// import {encrypt} from './encrypt';
let systemInfo = Taro.getSystemInfoSync();
let env = Taro.getEnv();

class Request {

  constructor() {
    this.genRequestMethod();
  }

  /**
   * 处理参数
   * @param {*} url
   * @param {*} options
   * @param {*} data
   */
  handleParams(url, options) {
    const {header = {}, ...rest} = options;

    let userState = getStore().getState().user;
    // console.log('userState.....', userState);
    const {token} = userState || {};

    if (!/http|https/.test(url)) {
      url = `${config.httpBaseUrl ? config.httpBaseUrl : ''}${url}`
    };

    let params = {
      url,
      timeout: 10 * 1000,
      header: {
        'Content-Type': 'application/json;charset=UTF-8',
        'token': token || '',
        ...header,
      },
      ...rest
    };

    if (!token) {
      delete params.header.token;
    }

    return params;
  }

  /**
   * 处理返回结果
   * @param {*} res
   * @param {*} resolve
   * @param {*} reject
   */
  handleResult(res, resolve, reject) {
    //防止后端返回的是一个字符串
    if (typeof res.data === 'string') {
      try {
        res.data = JSON.parse(res.data);
      } catch(err) {

      }
    }

    if (res.statusCode == 200 && res.data.success) {
      resolve(res.data);
    }

    if (res.statusCode == 200 && !res.data.success) {
      Taro.showToast({title: res.data.errorMsg, icon: 'none'});
      //未登陆
      if (res.data.errorCode == 403) {
        //如果跳过登录就不跳了
        //   Taro.navigateTo({url: '/pages/login/index?back=1'})
        // })
      }
      reject(res.data);
    }

    if (res.statusCode != 200) {
      Taro.showToast({title: '服务器异常', icon: 'none'});
      reject({
        errorCode: '1000000',
        errorMsg: '服务器异常'
      })
    }
  }

  /**
   * 生成请求方法
   */
  genRequestMethod() {
    ['get', 'post', 'delete', 'put'].forEach(methodName => {
      this[methodName] = (url, data = {}, options = {}) => {
        return new Promise((resolve, reject) => {
          let params = this.handleParams(url, options);
          params.timeout = 100 * 1000;
          params.method = methodName.toUpperCase();
          let timeStamp = (new Date()).getTime();

          const channel = CHANNEL[env];

          params.data = {
            appId: channel.id,
            batchNo: timeStamp,
            timestamp: timeStamp,
            charset: 'utf-8',
            format: 'JSON',
            signType: 'RSA2',
            channel: channel.name,
            // sign: 'xiaoya',
            deviceNo: timeStamp,
            ...data,
          };

          // if (url.indexOf('/login') > -1) {
          //   params.data.appletType = 2;
          // }
          // let sign = encrypt(params.data);

          params.data.sign = 'xiaoya';
          // params.data.signType = 'RSA2';


          if (methodName === 'post') {
            params.data = JSON.stringify(params.data || {});
          }

          // console.log(params.data);

          Taro.request(params)
            .then(res => {
              // console.log('res....', res);
              // try {
                this.handleResult(res, resolve, reject);
              // } catch(err) {
              //   console.log('ggggggg', err);
              // }

            })
            .catch(err => {
              // console.log('errr.......', err);
              reject({
                errorCode: '1000000',
                errorMsg: '服务器异常'
              })
            })
        })
      }
    })
  }

  /**
   * 上传文件
   */
  upload(url, options = {}) {
    if (typeof my !== 'undefined') {
      //做一下兼容，支付宝/微信文件名字段不一样
      if (options.name) {
        options.fileName = options.name;
        options.fileType = options.fileType || 'image';
        // delete options.name;
      }
    }

    let params = this.handleParams(url, options);
    return new Promise((resolve, reject) => {
      Taro.uploadFile(params)
        .then(res => {
          this.handleResult(res, resolve, reject);
        })
        .catch(err => {
          reject({
            errorCode: '1000000',
            errorMsg: '服务器异常'
          })
        })
    })
  }
}

export default new Request();
